package Stock

import (
	"ERP_storage/models/Basic"
	"git.baozhida.cn/ERP_libs/lib"
	orm2 "github.com/beego/beego/v2/client/orm"
	"strconv"
	"time"

	"ERP_storage/logs"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	_ "github.com/go-sql-driver/mysql"
)

// 入库产品mingxi
type StockOutProduct struct {
	Id            int       `orm:"column(ID);size(11);auto;pk"`
	T_number      string    `orm:"size(256);null"`                                        // 出库单号
	T_depot_id    int       `orm:"size(20);null"`                                         // 仓库id
	T_product_id  int       `orm:"size(20);null"`                                         // 产品id
	T_num         int       `orm:"size(20);null"`                                         // 入库数量
	T_date        string    `orm:"size(256);null"`                                        // 业务日期
	T_relation_sn string    `orm:"type(text);null"`                                       // 关联sn，20220101,20230202,
	CreateTime    time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime    time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *StockOutProduct) TableName() string {
	return "stock_out_product" // 数据库名称   // ************** 替换 FormulaList **************
}

type StockOutProductDaoImpl struct {
	orm orm.Ormer
}

func NewStockOutProduct(orm orm.Ormer) *StockOutProductDaoImpl {
	return &StockOutProductDaoImpl{orm: orm}
}

func init() {
	//注册模型
	orm.RegisterModel(new(StockOutProduct))
}

type StockOutProduct_R struct {
	Id            int
	T_number      string   // 入库单号
	T_depot_id    int      // 仓库id
	T_num         int      // 入库数量
	T_date        string   // 业务日期
	T_device_list []string // sn关联列表

	// ---------产品信息-----------
	T_product_id          int    // 产品id
	T_product_name        string // 产品名称
	T_product_class_name  string // 产品分类名称
	T_product_model       string // 产品型号
	T_product_spec        string // 产品规格
	T_product_relation_sn int    // 是否关联sn 0-否 1-是
	T_product_img         string // 产品图片
}

func StockOutProductToStockOutProduct_R(t StockOutProduct) (r StockOutProduct_R) {
	r.Id = t.Id
	r.T_number = t.T_number
	r.T_depot_id = t.T_depot_id
	r.T_num = t.T_num
	r.T_date = t.T_date
	//r.T_relation_sn = t.T_relation_sn
	r.T_device_list = lib.SplitString(t.T_relation_sn, ",")

	r.T_product_id = t.T_product_id
	product, _ := Basic.Read_Product_ById(t.T_product_id)
	r.T_product_name = product.T_name
	r.T_product_class_name = Basic.Read_ProductClass_Get(product.T_class)
	r.T_product_model = product.T_model
	r.T_product_spec = product.T_spec
	r.T_product_relation_sn = product.T_relation_sn
	r.T_product_img = product.T_img
	return r
}

// 添加
func (dao *StockOutProductDaoImpl) Add_StockOutProduct(r StockOutProduct) (id int64, err error) {
	id, err = dao.orm.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// 获取 ById
func (dao *StockOutProductDaoImpl) Read_StockOutProduct_List(T_number string) (r []StockOutProduct) {
	qs := dao.orm.QueryTable(new(StockOutProduct))
	_, err := qs.Filter("T_number", T_number).All(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 修改
func (dao *StockOutProductDaoImpl) Update_StockOutProduct(m StockOutProduct, cols ...string) error {
	_, err := dao.orm.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	return nil
}

// 根据库存id、产品id、月份获取本月出库总数量
func (dao *StockOutProductDaoImpl) Read_StockOut_Total(T_depot_id, T_product_id int, T_date string) int {
	sql := "SELECT SUM(t_num) FROM stock_out_product WHERE t_depot_id = " + strconv.Itoa(T_depot_id) + " AND t_product_id = " + strconv.Itoa(T_product_id) + " AND t_date like '" + T_date + "%'"

	var pl_lists orm2.ParamsList
	_, err := dao.orm.Raw(sql).ValuesFlat(&pl_lists)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return 0
	}
	if pl_lists[0] == nil {
		return 0
	}
	key, _ := strconv.Atoi(pl_lists[0].(string))
	return key
}
